@@ -34,6 +34,18 @@
 namespace reSID
 {
 
+inline short clip(int input)
+{
+    // Saturated arithmetics to guard against 16 bit sample overflow.
+    if (unlikely(input > 32767)) {
+      return 32767;
+    }
+    if (unlikely(input < -32768)) {
+      return -32768;
+    }
+    return (short)input;
+}
+
 // ----------------------------------------------------------------------------
 // Constructor.
 // ----------------------------------------------------------------------------
@@ -910,7 +922,7 @@ int SID::clock_resample(cycle_count& delta_t, short* buf, int n, int interleave)
 
     for (int i = 0; i < delta_t_sample; i++) {
       clock();
-      sample[sample_index] = sample[sample_index + RINGSIZE] = output();
+      sample[sample_index] = sample[sample_index + RINGSIZE] = clip(output());
       ++sample_index &= RINGMASK;
     }
 
@@ -953,16 +965,7 @@ int SID::clock_resample(cycle_count& delta_t, short* buf, int n, int interleave)
 
     v >>= FIR_SHIFT;
 
-    // Saturated arithmetics to guard against 16 bit sample overflow.
-    const int half = 1 << 15;
-    if (v >= half) {
-      v = half - 1;
-    }
-    else if (v < -half) {
-      v = -half;
-    }
-
-    buf[s*interleave] = v;
+    buf[s*interleave] = clip(v);
   }
 
   return s;
@@ -1009,16 +1012,7 @@ int SID::clock_resample_fastmem(cycle_count& delta_t, short* buf, int n, int int
 
     v >>= FIR_SHIFT;
 
-    // Saturated arithmetics to guard against 16 bit sample overflow.
-    const int half = 1 << 15;
-    if (v >= half) {
-      v = half - 1;
-    }
-    else if (v < -half) {
-      v = -half;
-    }
-
-    buf[s*interleave] = v;
+    buf[s*interleave] = clip(v);
   }
 
   return s;
